<?php
/**
 * @package     plg_easyshop_wishlist
 * @version     1.0.0
 * @Author      Vu Mai - www.joomtech.net
 * @copyright   Copyright (C) 2016 - 2017 All Rights Reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

defined('_JEXEC') or die;
JLoader::import('easyshop.plugin.dispatch');
JLoader::register('EasyshopHelperRoute', ES_COMPONENT_SITE . '/helpers/route.php');

use ES\Plugin\PluginDispatch;
use Joomla\Utilities\ArrayHelper;

class PlgEasyshopWishlist extends PluginDispatch
{
	protected $context = 'com_easyshop.wishlist';
	protected $taskMaps = ['wishlist'];
	/**
	 * @var $user \ES\User
	 * @since 1.0.0
	 */
	protected $user;

	public function __construct($subject, array $config = array())
	{
		parent::__construct($subject, $config);
		$this->user = easyshop('class.user');
		$this->user->load();
	}

	public function onEasyshopProductPrepare($product)
	{
		if ($this->params->get('allowGuests', 1)
			|| $this->params->get('stillShowButton', 0)
			|| $this->user->isCustomer()
		)
		{
			$extraDisplay = $this->getRenderer()->render('wishlist.button', [
				'productId' => $product->id,
				'items'     => $this->app->getUserState($this->context, []),
				'params'    => $this->params,
			]);

			if ($this->params->get('showOnBlock', 1))
			{
				$product->extraBlockFlexDisplay['wishlist'] = $extraDisplay;
			}

			if ($this->params->get('showOnDetail', 1))
			{
				$product->extraDetailFlexDisplay['wishlist'] = $extraDisplay;
			}
		}
	}

	protected function removeProduct($productId)
	{
		if ($this->user->isCustomer())
		{
			$userId = (int) $this->user->__get('id');
			$db     = easyshop('db');
			$query  = $db->getQuery(true)
				->delete($db->qn('#__easyshop_wishlist'))
				->where($db->qn('user_id') . ' = ' . $userId)
				->where($db->qn('product_id') . ' = ' . (int) $productId);
			$db->setQuery($query)
				->execute();
		}
	}

	public function onEasyshopLoadProductsFromWishlist($items = null)
	{
		if (null === $items)
		{
			$items = $this->app->getUserState($this->context, []);
		}

		$products = [];

		if ($items)
		{
			$productClass = easyshop('class.product');

			foreach ($items as &$item)
			{
				if (($product = $productClass->getItem($item, false, false))
					&& (int) $product->state == 1
				)
				{
					$products[] = $product;
				}
				else
				{
					$this->removeProduct($item);
					unset($item);
				}
			}
		}

		$this->app->setUserState($this->context, $items);

		return $products;
	}

	public function onAjaxWishlist()
	{
		$this->initialiseWishListItems();
		$productId   = (int) $this->app->input->post->getUint('productId', 0);
		$displayMode = (int) $this->params->get('resultDisplayMode', 1);
		$type        = $this->app->input->post->getWord('type', 'add');
		$items       = $this->app->getUserState($this->context, []);
		$product     = easyshop('class.product')->getItem($productId, false, false);
		$renderer    = $this->getRenderer();

		if (!$product)
		{
			throw new RuntimeException(JText::_('COM_EASYSHOP_PRODUCT_NOT_FOUND'));
		}

		if (!$this->params->get('allowGuests', 1)
			&& !$this->user->isCustomer()
		)
		{
			return $renderer->render('wishlist.modal.guest', [
				'product' => $product,
			]);
		}

		if ($type === 'add')
		{
			if (!in_array($productId, $items)
				&& (int) $product->state === 1
			)
			{
				$items[] = $productId;

				if ($this->user->isCustomer())
				{
					$userId = (int) $this->user->__get('id');
					JTable::addIncludePath(ES_PLUGIN_WISHLIST_ADMINISTRATOR . '/tables');
					$table = JTable::getInstance('Wishlist', 'EasyshopTable');
					$table->bind(['user_id' => $userId, 'product_id' => $productId]);
					$table->store();
				}
			}

			$items = array_unique($items);
		}
		else
		{
			$newItems = [];

			foreach ($items as $item)
			{
				if ($item != $productId)
				{
					$newItems[] = $item;
				}
			}

			$this->removeProduct($productId);
			$items = $newItems;
		}

		$products = $this->onEasyshopLoadProductsFromWishlist($items);

		if ($products)
		{
			$displayData = [
				'params'      => $this->params,
				'products'    => $products,
				'product'     => $product,
				'wishlistURL' => JRoute::_(EasyshopHelperRoute::getCustomViewRoute('wishlist'), false)
			];

			if ($displayMode === 1)
			{
				return $renderer->render('wishlist.modal.result', $displayData);
			}

			if ($displayMode === 2)
			{
				return $renderer->render('wishlist.modal.simple', $displayData);
			}

			return [
				'redirect' => $displayData['wishlistURL'],
			];
		}

		return $renderer->render('wishlist.modal.empty');
	}

	public function onEasyshopGetMenuTypeOptions(&$option, MenusModelMenutypes $model)
	{
		$o = new JObject;
		$o->set('title', 'PLG_EASYSHOP_WISHLIST_PAGE');
		$o->set('description', 'PLG_EASYSHOP_WISHLIST_PAGE_DESC');
		$o->set('request', [
			'option' => 'com_easyshop',
			'view'   => 'wishlist',
		]);
		$option[] = $o;
		$model->addReverseLookupUrl($o);
	}

	protected function initialiseWishListItems()
	{
		if ($this->user->isCustomer())
		{
			$items  = $this->app->getUserState($this->context, []);
			$userId = (int) $this->user->__get('id');
			$db     = easyshop('db');
			$query  = $db->getQuery(true)
				->select('a.product_id')
				->from($db->qn('#__easyshop_wishlist', 'a'))
				->where($db->qn('user_id') . ' = ' . $userId);
			$db->setQuery($query);

			if ($productIds = $db->loadColumn())
			{
				$items  = array_merge($items, ArrayHelper::toInteger($productIds));
				$items  = ArrayHelper::arrayUnique($items);
				$userId = (int) $this->user->__get('id');
				JTable::addIncludePath(ES_PLUGIN_WISHLIST_ADMINISTRATOR . '/tables');
				$table = JTable::getInstance('Wishlist', 'EasyshopTable');

				foreach ($items as $item)
				{
					$table->bind(['user_id' => $userId, 'product_id' => $item]);
					$table->store();
				}

				$this->app->setUserState($this->context, $items);
			}
		}
	}

	public function onEasyshopBeforeDispatch(&$config)
	{
		$this->initialiseWishListItems();
		parent::onEasyshopBeforeDispatch($config);
	}
}